"""
This module is called "cache" because it builds on the "cache" plugin:

    https://docs.pytest.org/en/latest/cache.html

"""

FAILED_FIRST_LAST_FAILED_BUCKET_KEY = '<failed_first_last_failed>'


def process_failed_first_last_failed(session, config, items):
    if not config.getoption('failedfirst'):
        return

    last_failed_raw = config.cache.get('cache/lastfailed', None)
    if not last_failed_raw:
        return

    # Get the names of last failed tests
    last_failed = []
    for key in last_failed_raw.keys():
        parts = key.split('::')
        if len(parts) == 3:
            last_failed.append(tuple(parts))
        elif len(parts) == 2:
            last_failed.append((parts[0], None, parts[1]))
        else:
            raise NotImplementedError()

    def assign_last_failed_to_same_bucket(item, key):
        if item.nodeid in last_failed_raw:
            return FAILED_FIRST_LAST_FAILED_BUCKET_KEY
        else:
            return key

    session.random_order_bucket_type_key_handlers.append(assign_last_failed_to_same_bucket)



import collections

import pytest

pytest_plugins = 'pytester'


Call = collections.namedtuple('Call', field_names=('package', 'module', 'cls', 'name'))


def _get_test_calls(result):
    """
    Returns a tuple of test calls in the order they were made.
    """
    calls = []

    for c in result.reprec.getcalls('pytest_runtest_call'):
        calls.append(Call(
            package=c.item.module.__package__,
            module=c.item.module.__name__,
            cls=(c.item.module.__name__, c.item.cls.__name__) if c.item.cls else None,
            name=c.item.name,
        ))
    return tuple(calls)


@pytest.fixture
def get_test_calls():
    """
    Returns a function to get runtest calls out from testdir.pytestrun result object.
    """
    return _get_test_calls


@pytest.fixture
def twenty_tests():
    code = []
    for i in range(20):
        code.append('def test_a{0}(): assert True\n'.format(str(i).zfill(2)))
    return ''.join(code)


@pytest.fixture
def twenty_cls_tests():
    code = []
    for i in range(20):
        code.append('\tdef test_b{0}(self): self.assertTrue\n'.format(str(i).zfill(2)))
    return ''.join(code)



# -*- coding: utf-8 -*-
import py
import pytest


@pytest.fixture
def tmp_tree_of_tests(testdir):
    """
    Creates a directory structure:
        tmpdir/
            foo.py

    """

    utils_package = testdir.mkpydir('utils')
    utils_package.join('__init__.py').write('')

    utils_package.join('foo.py').write(py.code.Source('''
        def add(a, b):
            """
            >>> add(1, 1)
            2
            >>> add(0, 0)
            0
            """
            return a + b

        def subtract(a, b):
            """
            >>> subtract(1, 1)
            0
            >>> subtract(0, 2)
            -2
            """
            return a - b
    '''))

    return testdir


@pytest.mark.parametrize('bucket', [
    'global',
    'package',
    'module',
    'class',
    'parent',
    'grandparent',
    'none',
])
def test_doctests(tmp_tree_of_tests, get_test_calls, bucket):
    result1 = tmp_tree_of_tests.runpytest(
        '--doctest-modules', '--random-order-bucket={0}'.format(bucket), '--verbose', '-s',
    )
    result1.assert_outcomes(passed=2, failed=0)
    assert 'PytestWarning' not in result1.stdout.str()
    assert 'PytestWarning' not in result1.stderr.str()



def test_help_message(testdir):
    result = testdir.runpytest(
        '--help',
    )
    result.stdout.fnmatch_lines([
        'pytest-random-order options:',
        '*--random-order-bucket={global,package,module,class,parent,grandparent,none}*',
        '*--random-order-seed=*',
    ])


def test_markers_message(testdir):
    result = testdir.runpytest(
        '--markers',
    )
    result.stdout.fnmatch_lines([
        '*@pytest.mark.random_order(disabled=True): disable reordering*',
    ])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import codecs
import os

from setuptools import setup


def read(fname):
    file_path = os.path.join(os.path.dirname(__file__), fname)
    return codecs.open(file_path, encoding='utf-8').read()


setup(
    name='pytest-random-order',
    version='1.1.0',
    author='Jazeps Basko',
    author_email='jazeps.basko@gmail.com',
    maintainer='Jazeps Basko',
    maintainer_email='jazeps.basko@gmail.com',
    license='MIT',
    url='https://github.com/jbasko/pytest-random-order',
    description='Randomise the order in which pytest tests are run with some control over the randomness',
    long_description=read('README.rst'),
    packages=[
        'random_order',
    ],
    include_package_data=True,
    python_requires=">=3.5.0",
    install_requires=[
        'pytest>=3.0.0',
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Pytest',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Testing',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'License :: OSI Approved :: MIT License',
    ],
    keywords='pytest random test order shuffle',
    entry_points={
        'pytest11': [
            'random_order = random_order.plugin',  # >=1.0.0
        ],
    },
)



class Config:

    @classmethod
    def default_value(cls, value):
        return "default:" + str(value)

    def __init__(self, config):
        self._config = config

    @property
    def bucket_type(self):
        if not self.is_enabled:
            return 'none'
        else:
            return self._remove_default_prefix(self._config.getoption('random_order_bucket'))

    @property
    def is_enabled(self):
        return (
            self._config.getoption('random_order_enabled') or
            any(
                not self._config.getoption(name).startswith('default:')
                for name in ('random_order_bucket', 'random_order_seed')
            )
        )

    @property
    def seed(self):
        return self._remove_default_prefix(self._config.getoption('random_order_seed'))

    def _remove_default_prefix(self, value):
        if value.startswith('default:'):
            return value[len('default:'):]
        return value
